@@ -20,7 +20,7 @@
 
 ***************************************************************************/
 
-//     zfuncs.h   version  v.4.1
+//     zfuncs.h   version  v.4.7
 
 #include <sys/time.h>
 #include <sys/stat.h>
@@ -36,7 +36,7 @@
 #include <signal.h>
 #include <execinfo.h>
 #include <locale.h>
-#include <sys/inotify.h>
+#include <sys/resource.h>
 
 #define  int8   char                                                       //  number types
 #define  int16  short
@@ -77,7 +77,11 @@ void start_timer(double &time0);                                           //  s
 double get_timer(double &time0);                                           //  get elapsed time in seconds
 void start_CPUtimer(double &time0);                                        //  start a process CPU timer
 double get_CPUtimer(double &time0);                                        //  get elapsed CPU time, seconds
-double CPUtime();                                                          //  get elapsed process CPU time used
+double CPUtime();                                                          //  get elapsed process CPU time for main()
+double CPUtime2();                                                         //   " include all threads 
+double jobtime();                                                          //   " include all threads + subprocesses
+int  parseprocfile(cchar *pfile, cchar *pname, double *value, ...);        //  get data from /proc file
+int  parseprocrec(char *prec, int field, double *value, ...);              //  get data from /proc file record
 void zsleep(double dsecs);                                                 //  sleep specified seconds
 
 void start_detached_thread(void * tfunc(void *), void * arg);              //  start detached thread function
@@ -87,6 +91,9 @@ int  zget_locked(int &param);                                              //  l
 void zput_locked(int &param, int value);                                   //  put and unlock multi-thread parameter
 int  zadd_locked(int &param, int incr);                                    //  increment multi-thread parameter
 
+void gate_threads_enter();                                                 //  allow only one thread at a time within
+void gate_threads_leave();                                                 //    the time betwen enter() and leave()
+
 char * zmalloc(size_t bytes, cchar *tag = 0);                              //  malloc() with auto crash and log option
 void zfree(void *buff);                                                    //  free() with log option
 void zmalloc_log(int count);                                               //  turn zmalloc() log on/off       v.3.5
@@ -100,7 +107,6 @@ int checkinstall(cchar *prog, ...);                                        //  c
 char * fgets_trim(char * buff, int maxcc, FILE *, int bf = 0);             //  fgets + trim trailing \n \r (blanks)
 int samedirk(cchar *file1, cchar *file2);                                  //  returns 1 if files in same directory
 int parsefile(cchar *path, char **dir, char **file, char **ext);           //  parse a filespec
-int zmondirk(cchar *action, cchar *dirk, char **file);                     //  monitor a directory for changes   v.2.16
 
 //  measure CPU time spent in a function or code block within a function
 extern volatile double   cpu_profile_timer;                                //  internal data tables
@@ -231,8 +237,8 @@ double spline2(double x);                                                  //  r
 
 //  application initialization and administration
 
-#define ZTXmaxent 1000                                                     //  max. translation strings
-#define ZTXmaxcc 2000                                                      //  max. cc per string
+#define ZTXmaxent 2000                                                     //  max. translation strings
+#define ZTXmaxcc 4000                                                      //  max. cc per string
 
 int zinitapp(cchar *appname, ...);                                         //  initz. app directories and files   v.4.1
 cchar * get_zinstalloc();                                                  //  get /usr or /usr/local  ...        v.4.1
@@ -244,10 +250,12 @@ void showz_userguide(cchar *context = 0);                                  //  s
 void showz_readme();                                                       //  show README file in popup window
 void showz_changelog();                                                    //  show changelog file in popup window
 void showz_translations();                                                 //  show TRANSLATIONS file in popup window
-void zmake_menu_launcher(cchar *categories, cchar *genericname);           //  add desktop menu and launcher      v.4.1
+void showz_doctext(cchar *file);                                           //  show text file (or .gz) in a popup window
+void zmake_menu_launcher(cchar *command, cchar *cats, cchar *generic);     //  add desktop menu and launcher      v.4.1
 
-int ZTXinit(cchar *lang);                                                  //  setup for message translation
+void ZTXinit(cchar *lang);                                                 //  setup for message translation
 cchar *ZTX(cchar *english);                                                //  get translation for English message
+void ZTX_translation_start();                                              //  start online translation
 
 /**************************************************************************
    GTK utility functions
@@ -311,6 +319,7 @@ struct zwidget  {
       int         homog;                           //  hbox/vbox: equal spacing flag
       int         expand;                          //  expandable flag
       int         space;                           //  extra padding space (pixels)
+      int         wrap;                            //  wrap mode for edit widget
       GtkWidget   *widget;                         //  GTK widget pointer
 };
 
@@ -319,6 +328,9 @@ struct zdialog  {
       void        *eventCB;                        //  widget event user callback function
       int         zstat;                           //  dialog status (from completion button)
       int         disabled;                        //  widget signals/events are disabled
+      int         saveposn;                        //  save and recall window position each use
+      cchar       *help_topic;                     //  optional help topic
+      GtkWidget   *parent;                         //  parent window or null
       GtkWidget   *compbutt[zdmaxbutts];           //  dialog completion buttons
       zwidget     widget[zdmaxwidgets];            //  dialog widgets (EOF = type = 0)
 };
@@ -328,20 +340,22 @@ zdialog *zdialog_new(cchar *title, GtkWidget *parent, ...);                //  c
 int zdialog_add_widget(zdialog *zd,                                        //  add widget to zdialog
       cchar *type, cchar *name, cchar *pname,                              //  required args
       cchar *data = 0, int scc = 0, int homog = 0,                         //  optional args
-      int expand = 0, int space = 0);
+      int expand = 0, int space = 0, int wrap = 0);
 
 int zdialog_add_widget(zdialog *zd,                                        //  add widget to zdialog
       cchar *type, cchar *name, cchar *pname,                              //  (alternative form)
-      cchar *data, cchar *options);                                        //  "scc=nn|homog|expand|space=nn"
+      cchar *data, cchar *options);                                        //  "scc=nn|homog|expand|space=nn|wrap"
 
 GtkWidget * zdialog_widget(zdialog *zd, cchar *name);                      //  GTK widget from zdialog widget name
 int zdialog_set_group(zdialog *zd, cchar *radio1, ...);                    //  set group ID for set of radio buttons
 int zdialog_resize(zdialog *zd, int width, int height);                    //  set size > widget sizes
 int zdialog_put_data(zdialog *zd, cchar *name, cchar *data);               //  put data in widget (entry, spin ...)
 cchar * zdialog_get_data(zdialog *zd, cchar *name);                        //  get widget data
+int zdialog_set_limits(zdialog *zd, cchar *name, double min, double max);  //  set new widget limits (spin, scale)
+void zdialog_help(zdialog *zd, cchar *help_topic);                         //  set help topic for zdialog
 
 typedef int zdialog_event(zdialog *zd, cchar *name);                       //  widget event callback function
-int zdialog_run(zdialog *zd, zdialog_event = 0);                           //  run dialog, handle events
+int zdialog_run(zdialog *zd, zdialog_event = 0, cchar *posn = 0);          //  run dialog, handle events
 
 int zdialog_send_event(zdialog *zd, cchar *event);                         //  send an event to an active dialog   v.2.17
 int zdialog_send_response(zdialog *zd, int zstat);                         //  complete a dialog, set status   v.2.23
@@ -367,6 +381,10 @@ int zdialog_cb_delete(zdialog *zd, cchar *name, cchar *data);              //  d
 int zdialog_cb_clear(zdialog *zd, cchar *name);                            //  clear all combo box entries         v.2.4
 int zdialog_cb_popup(zdialog *zd, cchar *name);                            //  show all combo box list entries     v.3.8
 
+int  zdialog_positions(cchar *action);                                     //  load or save zdialog window positions
+void zdialog_set_position(zdialog *zd, cchar *posn);                       //  set initial/new zdialog window position
+void zdialog_save_position(zdialog *zd);                                   //  save zdialog window position
+
 /**************************************************************************/
 
 //   GTK misc. utility functioins
@@ -378,8 +396,8 @@ int popup_command(cchar *command, int ww = 400, int hh = 300, GtkWidget *parent
 void zmessageACK(GtkWidget *parent, cchar *pMess, ... );                   //  display message, wait for OK
 void zmessLogACK(GtkWidget *parent, cchar *pMess, ... );                   //  same, with log to STDOUT
 int zmessageYN(GtkWidget *parent, cchar *pMess, ... );                     //  display message, wait for YES or NO
-void zmessage_help(GtkWidget *parent, cchar *topic, cchar *mess, ...);     //  message box, help button > user guide topic
-zdialog * zmessage_post(GtkWidget *parent, cchar *pMess, ...);             //  display message until killed    v.3.7
+void zmessage_help(GtkWidget *parent, cchar *topic, cchar *mess, ...);     //  message box, [help] > user guide topic
+zdialog * zmessage_post(GtkWidget *parent, int secs, cchar *mess, ...);    //  display message until timeout or cancel
 char * zdialog_text(GtkWidget *parent, cchar * title, cchar * initext);    //  get short text input from user
 
 //  file chooser dialogs for 1 file or multiple files
@@ -398,7 +416,10 @@ GdkCursor * zmakecursor(cchar *iconfile);                                  //  m
 //   GDK/GTK image file utility functions
 //   for functions returning char *, caller responsible for zfree()
 
-void print_imagefile(cchar *file, cchar *format = 0, cchar *orien = 0);    //  print image file (.jpg .png .tif)  v.3.4
+void print_image_paper_setup();                                            //  paper setup for printing an image file
+void print_image_margins_setup();                                          //  setup print margins
+void print_image_file(cchar *imagefile);                                   //  print the image file
+
 GdkPixbuf * gdk_pixbuf_rotate(GdkPixbuf *, double deg, int alfa = 0);      //  rotate pixbuf through any angle
 
 char * image_gallery(cchar *filez, cchar *action, int Nth = 0,             //  display image gallery window, navigate, 
@@ -406,8 +427,8 @@ char * image_gallery(cchar *filez, cchar *action, int Nth = 0,             //  d
 
 int image_gallery_position(cchar *file, int Nth);                          //  get relative position of file in gallery
 int image_file_type(cchar *file);                                          //  determine if directory or image file type
-char * image_thumbfile(cchar *imagefile);                                  //  get thumbnail filespec, create if missing
-GdkPixbuf * image_thumbnail(cchar *imagefile, int size = 0);               //  get thumbnail pixbuf, create if missing
+char * image_thumbfile(char *imagefile);                                   //  get thumbnail filespec, create if missing
+GdkPixbuf * image_thumbnail(char *imagefile, int size = 0);                //  get thumbnail pixbuf, create if missing
 
 char ** image_gallery_getfiles(char *startdir, GtkWidget *parent = 0);     //  select files from gallery window
 
